@@ -1,18 +1,17 @@
 /*
- * $Id$
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
  */
-
 package org.mule;
 
-import org.mule.api.MessagingException;
+import static org.mule.api.config.MuleProperties.OBJECT_EXPRESSION_LANGUAGE;
+import static org.mule.api.config.MuleProperties.OBJECT_POLLING_CONTROLLER;
+import static org.mule.api.config.MuleProperties.OBJECT_TRANSACTION_MANAGER;
+
+import org.mule.api.Injector;
 import org.mule.api.MuleContext;
-import org.mule.api.MuleEvent;
 import org.mule.api.MuleException;
 import org.mule.api.MuleRuntimeException;
 import org.mule.api.SingleResourceTransactionFactoryManager;
@@ -22,6 +21,7 @@ import org.mule.api.config.MuleProperties;
 import org.mule.api.config.ThreadingProfile;
 import org.mule.api.context.MuleContextAware;
 import org.mule.api.context.WorkManager;
+import org.mule.api.context.notification.FlowTraceManager;
 import org.mule.api.context.notification.ServerNotification;
 import org.mule.api.context.notification.ServerNotificationListener;
 import org.mule.api.el.ExpressionLanguage;
@@ -29,6 +29,7 @@ import org.mule.api.endpoint.EndpointFactory;
 import org.mule.api.exception.MessagingExceptionHandler;
 import org.mule.api.exception.RollbackSourceCallback;
 import org.mule.api.exception.SystemExceptionHandler;
+import org.mule.api.execution.ExceptionContextProvider;
 import org.mule.api.expression.ExpressionManager;
 import org.mule.api.lifecycle.Disposable;
 import org.mule.api.lifecycle.Initialisable;
@@ -40,10 +41,15 @@ import org.mule.api.registry.MuleRegistry;
 import org.mule.api.registry.RegistrationException;
 import org.mule.api.registry.Registry;
 import org.mule.api.security.SecurityManager;
+import org.mule.api.serialization.ObjectSerializer;
 import org.mule.api.store.ListableObjectStore;
-import org.mule.api.transaction.TransactionManagerFactory;
+import org.mule.api.store.ObjectStoreManager;
+import org.mule.api.util.StreamCloserService;
 import org.mule.client.DefaultLocalMuleClient;
+import org.mule.config.ClusterConfiguration;
 import org.mule.config.DefaultMuleConfiguration;
+import org.mule.config.NullClusterConfiguration;
+import org.mule.config.bootstrap.ArtifactType;
 import org.mule.config.i18n.CoreMessages;
 import org.mule.context.notification.MuleContextNotification;
 import org.mule.context.notification.NotificationException;
@@ -51,6 +57,7 @@ import org.mule.context.notification.ServerNotificationManager;
 import org.mule.exception.DefaultMessagingExceptionStrategy;
 import org.mule.exception.DefaultSystemExceptionStrategy;
 import org.mule.expression.DefaultExpressionManager;
+import org.mule.extension.ExtensionManager;
 import org.mule.lifecycle.MuleContextLifecycleManager;
 import org.mule.management.stats.AllStatistics;
 import org.mule.management.stats.ProcessingTimeWatcher;
@@ -66,6 +73,7 @@ import org.mule.util.ServerStartupSplashScreen;
 import org.mule.util.SplashScreen;
 import org.mule.util.SystemUtils;
 import org.mule.util.UUID;
+import org.mule.util.concurrent.Latch;
 import org.mule.util.lock.LockFactory;
 import org.mule.util.queue.QueueManager;
 
@@ -74,6 +82,7 @@ import java.util.Collection;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.Set;
+import java.util.concurrent.TimeUnit;
 
 import javax.resource.spi.work.WorkListener;
 import javax.transaction.TransactionManager;
@@ -84,6 +93,17 @@ import org.apache.commons.logging.LogFactory;
 
 public class DefaultMuleContext implements MuleContext
 {
+
+    /**
+     * TODO: Remove these constants.
+     * These constants only make sense until we have a reliable solution for durable persistence in Clustering.
+     * These are not part of Mule's API and you should not use them in applications or extensions
+     */
+    public static final String LOCAL_TRANSIENT_OBJECT_STORE_KEY = "_localInMemoryObjectStore";
+    public static final String LOCAL_PERSISTENT_OBJECT_STORE_KEY = "_localPersistentObjectStore";
+    public static final String LOCAL_OBJECT_STORE_MANAGER_KEY = "_localObjectStoreManager";
+    public static final String LOCAL_QUEUE_MANAGER_KEY = "_localQueueManager";
+
     /**
      * logger used by this class
      */
@@ -99,6 +119,13 @@ public class DefaultMuleContext implements MuleContext
      */
     private MuleRegistry muleRegistryHelper;
 
+    /**
+     * Default component to perform dependency injection
+     *
+     * @since 3.7.0
+     */
+    private Injector injector;
+
     /**
      * stats used for management
      */
@@ -125,29 +152,57 @@ public class DefaultMuleContext implements MuleContext
 
     private ExpressionManager expressionManager;
 
+    private volatile StreamCloserService streamCloserService;
+    private Object streamCloserServiceLock = new Object();
+
     private ClassLoader executionClassLoader;
 
     protected LocalMuleClient localMuleClient;
 
-    /** Global exception handler which handles "system" exceptions (i.e., when no message is involved). */
+    /**
+     * Global exception handler which handles "system" exceptions (i.e., when no message is involved).
+     */
     protected SystemExceptionHandler exceptionListener;
 
-    private String clusterId = "";
-
-    private int clusterNodeId;
-
     private PollingController pollingController = new DefaultPollingController();
 
-    private Map<QName, Set<Object>> configurationAnnotations;
+    private ClusterConfiguration clusterConfiguration = new NullClusterConfiguration();
+
+    private Map<QName, Set<Object>> configurationAnnotations = new HashMap<QName, Set<Object>>();
 
     private SingleResourceTransactionFactoryManager singleResourceTransactionFactoryManager = new SingleResourceTransactionFactoryManager();
 
-    private TransactionManager transactionManager;
-
     private LockFactory lockFactory;
 
+    private ExpressionLanguage expressionLanguage;
+
     private ProcessingTimeWatcher processingTimeWatcher;
 
+    private final Latch startLatch = new Latch();
+
+    private QueueManager queueManager;
+
+    private ExtensionManager extensionManager;
+
+    private ObjectSerializer objectSerializer;
+    private volatile DataTypeConversionResolver dataTypeConversionResolver;
+    private Object dataTypeConversionResolverLock = new Object();
+
+    private volatile FlowTraceManager flowTraceManager;
+    private Object flowTraceManagerLock = new Object();
+
+    private volatile Collection<ExceptionContextProvider> exceptionContextProviders;
+    private Object exceptionContextProvidersLock = new Object();
+
+    /**
+     * The {@link ArtifactType} indicating if this configuration object is for an application or a domain.
+     */
+    private ArtifactType artifactType;
+
+    /**
+     * @deprecated Use empty constructor instead and use setter for dependencies.
+     */
+    @Deprecated
     public DefaultMuleContext(MuleConfiguration config,
                               WorkManager workManager,
                               WorkListener workListener,
@@ -168,7 +223,10 @@ public class DefaultMuleContext implements MuleContext
         muleRegistryHelper = createRegistryHelper(registryBroker);
         localMuleClient = new DefaultLocalMuleClient(this);
         exceptionListener = new DefaultSystemExceptionStrategy(this);
-        configurationAnnotations = new HashMap<QName, Set<Object>>();
+    }
+
+    public DefaultMuleContext()
+    {
     }
 
     protected DefaultRegistryBroker createRegistryBroker()
@@ -181,6 +239,7 @@ public class DefaultMuleContext implements MuleContext
         return new MuleRegistryHelper(registry, this);
     }
 
+    @Override
     public synchronized void initialise() throws InitialisationException
     {
         lifecycleManager.checkPhase(Initialisable.PHASE_NAME);
@@ -194,15 +253,15 @@ public class DefaultMuleContext implements MuleContext
         {
             throw new MuleRuntimeException(CoreMessages.objectIsNull("workManager"));
         }
-        
+
         try
         {
-        	JdkVersionUtils.validateJdk();
+            JdkVersionUtils.validateJdk();
         }
         catch (RuntimeException e)
         {
-        	throw new InitialisationException(CoreMessages.invalidJdk(SystemUtils.JAVA_VERSION, 
-        			SystemUtils.JAVA_VENDOR, JdkVersionUtils.getSupportedJdks()), this);
+            throw new InitialisationException(CoreMessages.invalidJdk(SystemUtils.JAVA_VERSION,
+                                                                      JdkVersionUtils.getSupportedJdks()), this);
         }
 
         try
@@ -236,14 +295,11 @@ public class DefaultMuleContext implements MuleContext
         }
     }
 
+    @Override
     public synchronized void start() throws MuleException
     {
         getLifecycleManager().checkPhase(Startable.PHASE_NAME);
 
-        if (getSecurityManager() == null)
-        {
-            throw new MuleRuntimeException(CoreMessages.objectIsNull("securityManager"));
-        }
         if (getQueueManager() == null)
         {
             throw new MuleRuntimeException(CoreMessages.objectIsNull("queueManager"));
@@ -253,10 +309,13 @@ public class DefaultMuleContext implements MuleContext
 
         fireNotification(new MuleContextNotification(this, MuleContextNotification.CONTEXT_STARTING));
         getLifecycleManager().fireLifecycle(Startable.PHASE_NAME);
-
+        overridePollingController();
+        overrideClusterConfiguration();
 
         fireNotification(new MuleContextNotification(this, MuleContextNotification.CONTEXT_STARTED));
 
+        startLatch.release();
+
         if (logger.isInfoEnabled())
         {
             SplashScreen startupScreen = buildStartupSplash();
@@ -270,14 +329,18 @@ public class DefaultMuleContext implements MuleContext
      *
      * @throws MuleException if either any of the sessions or connectors fail to stop
      */
+    @Override
     public synchronized void stop() throws MuleException
     {
+        startLatch.release();
+
         getLifecycleManager().checkPhase(Stoppable.PHASE_NAME);
         fireNotification(new MuleContextNotification(this, MuleContextNotification.CONTEXT_STOPPING));
         getLifecycleManager().fireLifecycle(Stoppable.PHASE_NAME);
         fireNotification(new MuleContextNotification(this, MuleContextNotification.CONTEXT_STOPPED));
     }
 
+    @Override
     public synchronized void dispose()
     {
         if (isStarted())
@@ -312,7 +375,7 @@ public class DefaultMuleContext implements MuleContext
 
         notificationManager.dispose();
         workManager.dispose();
-        
+
         if (expressionManager != null && expressionManager instanceof Disposable)
         {
             ((Disposable) expressionManager).dispose();
@@ -334,6 +397,7 @@ public class DefaultMuleContext implements MuleContext
      *
      * @return true if the server has been initialised
      */
+    @Override
     public boolean isInitialised()
     {
         return getLifecycleManager().getState().isInitialised();
@@ -344,16 +408,19 @@ public class DefaultMuleContext implements MuleContext
      *
      * @return true if the server is beening initialised
      */
+    @Override
     public boolean isInitialising()
     {
         return getLifecycleManager().getState().isInitialising();
     }
 
+    @Override
     public boolean isStopped()
     {
         return getLifecycleManager().getState().isStopped();
     }
 
+    @Override
     public boolean isStopping()
     {
         return getLifecycleManager().getState().isStopping();
@@ -364,26 +431,31 @@ public class DefaultMuleContext implements MuleContext
      *
      * @return true if the server has been started
      */
+    @Override
     public boolean isStarted()
     {
         return getLifecycleManager().isPhaseComplete(Startable.PHASE_NAME);
     }
 
+    @Override
     public boolean isStarting()
     {
         return getLifecycleManager().getState().isStarting();
     }
 
+    @Override
     public boolean isDisposed()
     {
         return getLifecycleManager().getState().isDisposed();
     }
 
+    @Override
     public boolean isDisposing()
     {
         return getLifecycleManager().getState().isDisposing();
     }
 
+    @Override
     public LifecycleManager getLifecycleManager()
     {
         return lifecycleManager;
@@ -394,16 +466,19 @@ public class DefaultMuleContext implements MuleContext
      *
      * @return all statistics for this instance
      */
+    @Override
     public AllStatistics getStatistics()
     {
         return stats;
     }
 
+    @Override
     public void registerListener(ServerNotificationListener l) throws NotificationException
     {
         registerListener(l, null);
     }
 
+    @Override
     public void registerListener(ServerNotificationListener l, String resourceIdentifier) throws NotificationException
     {
         ServerNotificationManager notificationManager = getNotificationManager();
@@ -414,6 +489,7 @@ public class DefaultMuleContext implements MuleContext
         notificationManager.addListenerSubscription(l, resourceIdentifier);
     }
 
+    @Override
     public void unregisterListener(ServerNotificationListener l)
     {
         ServerNotificationManager notificationManager = getNotificationManager();
@@ -433,6 +509,7 @@ public class DefaultMuleContext implements MuleContext
      * @throws UnsupportedOperationException if the notification fired is not a
      *                                       {@link org.mule.context.notification.CustomNotification}
      */
+    @Override
     public void fireNotification(ServerNotification notification)
     {
         ServerNotificationManager notificationManager = getNotificationManager();
@@ -454,6 +531,7 @@ public class DefaultMuleContext implements MuleContext
      *                        authenticate and authorise incoming and outgoing event traffic
      *                        and service invocations
      */
+    @Override
     public void setSecurityManager(SecurityManager securityManager) throws RegistrationException
     {
         checkLifecycleForPropertySet(MuleProperties.OBJECT_SECURITY_MANAGER, Initialisable.PHASE_NAME);
@@ -468,9 +546,10 @@ public class DefaultMuleContext implements MuleContext
      *         and authorise incoming and outgoing event traffic and service
      *         invocations
      */
+    @Override
     public SecurityManager getSecurityManager()
     {
-        SecurityManager securityManager = (SecurityManager) registryBroker.lookupObject(MuleProperties.OBJECT_SECURITY_MANAGER);
+        SecurityManager securityManager = registryBroker.lookupObject(MuleProperties.OBJECT_SECURITY_MANAGER);
         if (securityManager == null)
         {
             Collection temp = registryBroker.lookupObjects(SecurityManager.class);
@@ -479,6 +558,10 @@ public class DefaultMuleContext implements MuleContext
                 securityManager = ((SecurityManager) temp.iterator().next());
             }
         }
+        if (securityManager == null)
+        {
+            throw new MuleRuntimeException(CoreMessages.objectIsNull("securityManager"));
+        }
         return securityManager;
     }
 
@@ -499,46 +582,104 @@ public class DefaultMuleContext implements MuleContext
      * @see org.mule.api.config.ThreadingProfile
      * @see DefaultMuleConfiguration
      */
+    @Override
     public WorkManager getWorkManager()
     {
         return workManager;
     }
 
+    @Override
     public WorkListener getWorkListener()
     {
         return workListener;
     }
 
+    @Override
     public QueueManager getQueueManager()
     {
-        QueueManager queueManager = (QueueManager) registryBroker.lookupObject(MuleProperties.OBJECT_QUEUE_MANAGER);
         if (queueManager == null)
         {
-            Collection<QueueManager> temp = registryBroker.lookupObjects(QueueManager.class);
-            if (temp.size() > 0)
+            queueManager = registryBroker.lookupObject(MuleProperties.OBJECT_QUEUE_MANAGER);
+            if (queueManager == null)
             {
-                queueManager = temp.iterator().next();
+                Collection<QueueManager> temp = registryBroker.lookupObjects(QueueManager.class);
+                if (temp.size() > 0)
+                {
+                    queueManager = temp.iterator().next();
+                }
             }
         }
         return queueManager;
     }
 
+    @Override
+    public ExtensionManager getExtensionManager()
+    {
+        return extensionManager;
+    }
+
+    @Override
+    public ObjectStoreManager getObjectStoreManager()
+    {
+        return this.getRegistry().lookupObject(MuleProperties.OBJECT_STORE_MANAGER);
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public ObjectSerializer getObjectSerializer()
+    {
+        return objectSerializer;
+    }
+
+    /**
+     * When running in clustered mode, it returns a {@link org.mule.api.store.ObjectStoreManager} that
+     * creates {@link org.mule.api.store.ObjectStore} instances which are only local to the current node.
+     * This is just a workaround until we introduce a solution for durable persistent stores in HA. This is not part of
+     * Mule's API and you should not use this in your apps or extensions
+     *
+     * @return a {@link org.mule.api.store.ObjectStoreManager}
+     * @since 3.5.0
+     */
+    public ObjectStoreManager getLocalObjectStoreManager()
+    {
+        return this.getRegistry().lookupObject(LOCAL_OBJECT_STORE_MANAGER_KEY);
+    }
+
+    /**
+     * When running in clustered mode, it returns a {@link org.mule.util.queue.QueueManager} that
+     * creates {@link org.mule.util.queue.Queue} instances which are only local to the current node.
+     * This is just a workaround until we introduce a solution for durable persistent queues in HA. This is not part of
+     * Mule's API and you should not use this in your apps or extensions
+     *
+     * @return a {@link org.mule.util.queue.QueueManager}
+     * @since 3.5.0
+     */
+    public QueueManager getLocalQueueManager()
+    {
+        return this.getRegistry().lookupObject(LOCAL_QUEUE_MANAGER_KEY);
+    }
+
+    @Override
     public void setQueueManager(QueueManager queueManager) throws RegistrationException
     {
-        checkLifecycleForPropertySet(MuleProperties.OBJECT_QUEUE_MANAGER, Initialisable.PHASE_NAME);
-        registryBroker.registerObject(MuleProperties.OBJECT_QUEUE_MANAGER, queueManager);
+        getRegistry().registerObject(MuleProperties.OBJECT_QUEUE_MANAGER, queueManager);
+        this.queueManager = queueManager;
     }
 
     /**
      * @return the MuleConfiguration for this MuleManager. This object is immutable
      *         once the manager has initialised.
      */
+    @Override
     public MuleConfiguration getConfiguration()
     {
 
         return config;
     }
 
+    @Override
     public ServerNotificationManager getNotificationManager()
     {
         return notificationManager;
@@ -550,10 +691,11 @@ public class DefaultMuleContext implements MuleContext
      * @param manager the manager to use
      * @throws RegistrationException if a transaction manager has already been set
      */
+    @Override
     public void setTransactionManager(TransactionManager manager) throws RegistrationException
     {
         //checkLifecycleForPropertySet(MuleProperties.OBJECT_TRANSACTION_MANAGER, Initialisable.PHASE_NAME);
-        registryBroker.registerObject(MuleProperties.OBJECT_TRANSACTION_MANAGER, manager);
+        registryBroker.registerObject(OBJECT_TRANSACTION_MANAGER, manager);
     }
 
     /**
@@ -563,36 +705,10 @@ public class DefaultMuleContext implements MuleContext
      * @return the Jta transaction manager used by this Mule server instance. or
      *         null if a transaction manager has not been set
      */
+    @Override
     public TransactionManager getTransactionManager()
     {
-        if (transactionManager == null)
-        {
-            transactionManager = (TransactionManager) registryBroker.lookupObject(MuleProperties.OBJECT_TRANSACTION_MANAGER);
-            if (transactionManager == null)
-            {
-                Collection temp = registryBroker.lookupObjects(TransactionManagerFactory.class);
-                if (temp.size() > 0)
-                {
-                    try
-                    {
-                        transactionManager = (((TransactionManagerFactory) temp.iterator().next()).create(config));
-                    }
-                    catch (Exception e)
-                    {
-                        throw new MuleRuntimeException(CoreMessages.failedToCreate("transaction manager"), e);
-                    }
-                }
-                else
-                {
-                    temp = registryBroker.lookupObjects(TransactionManager.class);
-                    if (temp.size() > 0)
-                    {
-                        transactionManager = (((TransactionManager) temp.iterator().next()));
-                    }
-                }
-            }
-        }
-        return transactionManager;
+        return getRegistry().lookupObject(OBJECT_TRANSACTION_MANAGER);
     }
 
     protected void checkLifecycleForPropertySet(String propertyName, String phase) throws IllegalStateException
@@ -603,31 +719,70 @@ public class DefaultMuleContext implements MuleContext
         }
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @Override
     public MuleRegistry getRegistry()
     {
         return muleRegistryHelper;
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public Injector getInjector()
+    {
+        return injector;
+    }
+
+    @Override
     public ThreadingProfile getDefaultMessageDispatcherThreadingProfile()
     {
         return (ThreadingProfile) getRegistry().lookupObject(MuleProperties.OBJECT_DEFAULT_MESSAGE_DISPATCHER_THREADING_PROFILE);
     }
 
+    @Override
     public ThreadingProfile getDefaultMessageRequesterThreadingProfile()
     {
         return (ThreadingProfile) getRegistry().lookupObject(MuleProperties.OBJECT_DEFAULT_MESSAGE_REQUESTER_THREADING_PROFILE);
     }
 
+    @Override
     public ThreadingProfile getDefaultMessageReceiverThreadingProfile()
     {
         return (ThreadingProfile) getRegistry().lookupObject(MuleProperties.OBJECT_DEFAULT_MESSAGE_RECEIVER_THREADING_PROFILE);
     }
 
+    @Override
     public ThreadingProfile getDefaultServiceThreadingProfile()
     {
         return (ThreadingProfile) getRegistry().lookupObject(MuleProperties.OBJECT_DEFAULT_SERVICE_THREADING_PROFILE);
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public StreamCloserService getStreamCloserService()
+    {
+        if (this.streamCloserService == null)
+        {
+            synchronized (streamCloserServiceLock)
+            {
+                if (this.streamCloserService == null)
+                {
+                    this.streamCloserService = this.getRegistry().lookupObject(
+                            MuleProperties.OBJECT_MULE_STREAM_CLOSER_SERVICE);
+                }
+            }
+        }
+
+        return this.streamCloserService;
+    }
+
+    @Override
     public ThreadingProfile getDefaultThreadingProfile()
     {
         return (ThreadingProfile) getRegistry().lookupObject(MuleProperties.OBJECT_DEFAULT_THREADING_PROFILE);
@@ -638,6 +793,7 @@ public class DefaultMuleContext implements MuleContext
      *
      * @return the long date when the server was started
      */
+    @Override
     public long getStartDate()
     {
         return startDate;
@@ -649,26 +805,39 @@ public class DefaultMuleContext implements MuleContext
      * @return the Expression Manager configured for this instance of Mule
      * @see org.mule.api.expression.ExpressionManager
      */
+    @Override
     public ExpressionManager getExpressionManager()
     {
         return expressionManager;
     }
 
+    @Override
     public void setExecutionClassLoader(ClassLoader cl)
     {
         this.executionClassLoader = cl;
     }
 
+    @Override
     public ClassLoader getExecutionClassLoader()
     {
         return executionClassLoader;
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    @Deprecated
     public void addRegistry(Registry registry)
     {
         registryBroker.addRegistry(registry);
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    @Deprecated
     public void removeRegistry(Registry registry)
     {
         registryBroker.removeRegistry(registry);
@@ -693,68 +862,59 @@ public class DefaultMuleContext implements MuleContext
         return shutdownScreen;
     }
 
+    @Override
     public LocalMuleClient getClient()
     {
         return localMuleClient;
     }
 
+    @Override
     public void handleException(Exception e, RollbackSourceCallback rollbackMethod)
     {
-        if (e instanceof MessagingException)
-        {
-            MuleEvent event = ((MessagingException) e).getEvent();
-            event.getFlowConstruct().getExceptionListener().handleException(e, event);
-        }
-        else
-        {
-            getExceptionListener().handleException(e, rollbackMethod);
-        }
+        getExceptionListener().handleException(e, rollbackMethod);
     }
 
+    @Override
     public void handleException(Exception e)
     {
         handleException(e, null);
     }
 
+    @Override
     public SystemExceptionHandler getExceptionListener()
     {
         return exceptionListener;
     }
 
+    @Override
     public void setExceptionListener(SystemExceptionHandler exceptionListener)
     {
         this.exceptionListener = exceptionListener;
     }
 
+    @Override
     public EndpointFactory getEndpointFactory()
     {
         return (EndpointFactory) registryBroker.lookupObject(MuleProperties.OBJECT_MULE_ENDPOINT_FACTORY);
     }
 
+    @Override
     public void setObjectStore(String name, ListableObjectStore<Serializable> store) throws RegistrationException
     {
         checkLifecycleForPropertySet(name, Initialisable.PHASE_NAME);
         registryBroker.registerObject(name, store);
     }
 
+    @Override
     public String getClusterId()
     {
-        return clusterId;
-    }
-
-    public void setClusterId(String clusterId)
-    {
-        this.clusterId = clusterId;
+        return clusterConfiguration.getClusterId();
     }
 
+    @Override
     public int getClusterNodeId()
     {
-        return clusterNodeId;
-    }
-
-    public void setClusterNodeId(int clusterNodeId)
-    {
-        this.clusterNodeId = clusterNodeId;
+        return clusterConfiguration.getClusterNodeId();
     }
 
     public void setPollingController(PollingController pollingController)
@@ -771,7 +931,7 @@ public class DefaultMuleContext implements MuleContext
     @Override
     public String getUniqueIdString()
     {
-        return clusterNodeId + "-" + UUID.getUUID();
+        return clusterConfiguration.getClusterNodeId() + "-" + UUID.getUUID();
     }
 
     @Override
@@ -808,19 +968,29 @@ public class DefaultMuleContext implements MuleContext
     @Override
     public DataTypeConversionResolver getDataTypeConverterResolver()
     {
-        DataTypeConversionResolver dataTypeConversionResolver = getRegistry().lookupObject(MuleProperties.OBJECT_CONVERTER_RESOLVER);
         if (dataTypeConversionResolver == null)
         {
-            dataTypeConversionResolver = new DynamicDataTypeConversionResolver(this);
+            synchronized (dataTypeConversionResolverLock)
+            {
+                if (dataTypeConversionResolver == null)
+                {
+                    dataTypeConversionResolver = getRegistry().lookupObject(MuleProperties.OBJECT_CONVERTER_RESOLVER);
 
-            try
-            {
-                getRegistry().registerObject(MuleProperties.OBJECT_CONVERTER_RESOLVER, dataTypeConversionResolver);
-            }
-            catch (RegistrationException e)
-            {
-                // Should not occur
-                throw new IllegalStateException(e);
+                    if (dataTypeConversionResolver == null)
+                    {
+                        dataTypeConversionResolver = new DynamicDataTypeConversionResolver(this);
+
+                        try
+                        {
+                            getRegistry().registerObject(MuleProperties.OBJECT_CONVERTER_RESOLVER, dataTypeConversionResolver);
+                        }
+                        catch (RegistrationException e)
+                        {
+                            // Should not occur
+                            throw new IllegalStateException(e);
+                        }
+                    }
+                }
             }
         }
 
@@ -830,7 +1000,12 @@ public class DefaultMuleContext implements MuleContext
     @Override
     public ExpressionLanguage getExpressionLanguage()
     {
-        return registryBroker.lookupObject(MuleProperties.OBJECT_EXPRESSION_LANGUAGE);
+        if (this.expressionLanguage == null)
+        {
+            this.expressionLanguage = this.registryBroker.lookupObject(OBJECT_EXPRESSION_LANGUAGE);
+        }
+
+        return this.expressionLanguage;
     }
 
     @Override
@@ -853,4 +1028,139 @@ public class DefaultMuleContext implements MuleContext
 
         return this.processingTimeWatcher;
     }
+
+    @Override
+    public boolean waitUntilStarted(int timeout) throws InterruptedException
+    {
+        return startLatch.await(timeout, TimeUnit.MILLISECONDS);
+    }
+
+    private void overrideClusterConfiguration()
+    {
+        ClusterConfiguration overriddenClusterConfiguration = getRegistry().get(MuleProperties.OBJECT_CLUSTER_CONFIGURATION);
+        if (overriddenClusterConfiguration != null)
+        {
+            this.clusterConfiguration = overriddenClusterConfiguration;
+        }
+    }
+
+    private void overridePollingController()
+    {
+        PollingController overriddenPollingController = getRegistry().get(OBJECT_POLLING_CONTROLLER);
+        if (overriddenPollingController != null)
+        {
+            this.pollingController = overriddenPollingController;
+        }
+    }
+
+    public void setMuleConfiguration(MuleConfiguration muleConfiguration)
+    {
+        this.config = muleConfiguration;
+    }
+
+    public void setWorkManager(WorkManager workManager)
+    {
+        this.workManager = workManager;
+    }
+
+    public void setworkListener(WorkListener workListener)
+    {
+        this.workListener = workListener;
+    }
+
+    public void setNotificationManager(ServerNotificationManager notificationManager)
+    {
+        this.notificationManager = notificationManager;
+    }
+
+    public void setLifecycleManager(MuleContextLifecycleManager lifecyleManager)
+    {
+        this.lifecycleManager = lifecyleManager;
+    }
+
+    public void setExpressionManager(DefaultExpressionManager expressionManager)
+    {
+        this.expressionManager = expressionManager;
+    }
+
+    public void setRegistryBroker(DefaultRegistryBroker registryBroker)
+    {
+        this.registryBroker = registryBroker;
+    }
+
+    public void setInjector(Injector injector)
+    {
+        this.injector = injector;
+    }
+
+    public void setMuleRegistry(MuleRegistryHelper muleRegistry)
+    {
+        this.muleRegistryHelper = muleRegistry;
+    }
+
+    public void setLocalMuleClient(DefaultLocalMuleClient localMuleContext)
+    {
+        this.localMuleClient = localMuleContext;
+    }
+
+    public void setExtensionManager(ExtensionManager extensionManager)
+    {
+        this.extensionManager = extensionManager;
+    }
+
+    public void setObjectSerializer(ObjectSerializer objectSerializer)
+    {
+        this.objectSerializer = objectSerializer;
+    }
+
+    @Override
+    public ArtifactType getArtifactType()
+    {
+        return artifactType;
+    }
+
+    public void setArtifactType(ArtifactType artifactType)
+    {
+        this.artifactType = artifactType;
+    }
+
+    @Override
+    public FlowTraceManager getFlowTraceManager()
+    {
+        if (flowTraceManager == null)
+        {
+            synchronized (flowTraceManagerLock)
+            {
+                if (flowTraceManager == null)
+                {
+                    try
+                    {
+                        flowTraceManager = getRegistry().lookupObject(FlowTraceManager.class);
+                    }
+                    catch (RegistrationException e)
+                    {
+                        // Should not occur
+                        throw new IllegalStateException(e);
+                    }
+                }
+            }
+        }
+        return flowTraceManager;
+    }
+
+    @Override
+    public Collection<ExceptionContextProvider> getExceptionContextProviders()
+    {
+        if (exceptionContextProviders == null)
+        {
+            synchronized (exceptionContextProvidersLock)
+            {
+                if (exceptionContextProviders == null)
+                {
+                    exceptionContextProviders = this.muleRegistryHelper.lookupByType(ExceptionContextProvider.class).values();
+                }
+            }
+        }
+        return exceptionContextProviders;
+    }
 }
