@@ -6,7 +6,7 @@
  */
 package org.mule.construct;
 
-import org.mule.api.AnnotatedObject;
+import org.mule.AbstractAnnotatedObject;
 import org.mule.api.MuleContext;
 import org.mule.api.MuleException;
 import org.mule.api.construct.FlowConstruct;
@@ -15,6 +15,7 @@ import org.mule.api.construct.FlowConstructInvalidException;
 import org.mule.api.context.MuleContextAware;
 import org.mule.api.exception.MessagingExceptionHandler;
 import org.mule.api.exception.MessagingExceptionHandlerAcceptor;
+import org.mule.api.exception.MessagingExceptionHandlerAware;
 import org.mule.api.lifecycle.Disposable;
 import org.mule.api.lifecycle.Initialisable;
 import org.mule.api.lifecycle.InitialisationException;
@@ -33,11 +34,6 @@ import org.mule.routing.MuleMessageInfoMapping;
 import org.mule.util.ClassUtils;
 
 import java.beans.ExceptionListener;
-import java.util.Collections;
-import java.util.Map;
-import java.util.concurrent.ConcurrentHashMap;
-
-import javax.xml.namespace.QName;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
@@ -60,7 +56,7 @@ import org.apache.commons.logging.LogFactory;
  * {@link #doStop()} and {@link #doDispose()} if they need to perform any action on
  * lifecycle transitions.
  */
-public abstract class AbstractFlowConstruct implements FlowConstruct, Lifecycle, AnnotatedObject
+public abstract class AbstractFlowConstruct extends AbstractAnnotatedObject implements FlowConstruct, Lifecycle
 {
     protected transient Log logger = LogFactory.getLog(getClass());
 
@@ -70,8 +66,7 @@ public abstract class AbstractFlowConstruct implements FlowConstruct, Lifecycle,
     protected final MuleContext muleContext;
     protected FlowConstructStatistics statistics;
     protected MessageInfoMapping messageInfoMapping = new MuleMessageInfoMapping();
-    private final Map<QName, Object> annotations = new ConcurrentHashMap<QName, Object>();
-    
+
     /**
      * The initial states that the flow can be started in
      */
@@ -283,7 +278,7 @@ public abstract class AbstractFlowConstruct implements FlowConstruct, Lifecycle,
         {
             if (!((MessagingExceptionHandlerAcceptor)exceptionListener).acceptsAll())
             {
-                throw new FlowConstructInvalidException(CoreMessages.createStaticMessage("Flow exception listener contains and exception strategy that doesn't handle all request," +
+                throw new FlowConstructInvalidException(CoreMessages.createStaticMessage("Flow exception listener contains an exception strategy that doesn't handle all request," +
                         " Perhaps there's an exception strategy with a when attribute set but it's not part of a catch exception strategy"),this);
             }
         }
@@ -301,6 +296,14 @@ public abstract class AbstractFlowConstruct implements FlowConstruct, Lifecycle,
         }
     }
 
+    protected void injectExceptionHandler(Object candidate)
+    {
+        if (candidate instanceof MessagingExceptionHandlerAware)
+        {
+            ((MessagingExceptionHandlerAware) candidate).setMessagingExceptionHandler(this.getExceptionListener());
+        }
+    }
+
     @Override
     public String toString()
     {
@@ -339,22 +342,6 @@ public abstract class AbstractFlowConstruct implements FlowConstruct, Lifecycle,
         }
     }
 
-    public final Object getAnnotation(QName name)
-    {
-        return annotations.get(name);
-    }
-
-    public final Map<QName, Object> getAnnotations()
-    {
-        return Collections.unmodifiableMap(annotations);
-    }
-
-    public synchronized final void setAnnotations(Map<QName, Object> newAnnotations)
-    {
-        annotations.clear();
-        annotations.putAll(newAnnotations);
-    }
-
     /**
      * @return the type of construct being created, e.g. "Flow"
      */
