@@ -77,13 +77,13 @@ public class DefaultEndpointFactory implements EndpointFactory
     public InboundEndpoint getInboundEndpoint(EndpointBuilder builder) throws MuleException
     {
         InboundEndpoint endpoint = builder.buildInboundEndpoint();
+        // Only continue to cache inbound endpoints because another project uses this.
         return (InboundEndpoint) registerEndpoint(endpoint);
     }
 
     public OutboundEndpoint getOutboundEndpoint(EndpointBuilder builder) throws MuleException
     {
-        OutboundEndpoint endpoint = builder.buildOutboundEndpoint();
-        return (OutboundEndpoint) registerEndpoint(endpoint);
+        return builder.buildOutboundEndpoint();
     }
 
     /**
@@ -123,7 +123,7 @@ public class DefaultEndpointFactory implements EndpointFactory
             logger.debug("Named EndpointBuilder not found, creating endpoint builder for uri");
             EndpointURI epURI = new MuleEndpointURI(uri, muleContext);
             TransportServiceDescriptor tsd = (TransportServiceDescriptor) muleContext.getRegistry().lookupServiceDescriptor(ServiceType.TRANSPORT, epURI.getFullScheme(), null);
-            endpointBuilder = tsd.createEndpointBuilder(uri);
+            endpointBuilder = tsd.createEndpointBuilder(uri, muleContext);
         }
         return endpointBuilder;
     }
