@@ -1,8 +1,5 @@
 /*
- * $Id: OutboundAttachmentsExpressionEvaluator.java 20321 2010-11-24 15:21:24Z dfeist $
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
@@ -10,7 +7,6 @@
 package org.mule.expression;
 
 import org.mule.api.MuleMessage;
-import org.mule.api.expression.ExpressionEvaluator;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
@@ -19,7 +15,7 @@ import org.apache.commons.logging.LogFactory;
  * Creates a Map facade around a {@link org.mule.api.MuleMessage} instance to allow access to outbound
  * attachments from within components and transformers without the these objects needing access to the Mule Message
  */
-public class OutboundAttachmentsExpressionEvaluator implements ExpressionEvaluator
+public class OutboundAttachmentsExpressionEvaluator extends AbstractExpressionEvaluator
 {
     public static final String NAME = "outboundAttachments";
 
