@@ -13,6 +13,7 @@ import org.mule.module.db.internal.domain.query.QueryTemplate;
 import org.mule.module.db.internal.domain.type.DbType;
 import org.mule.module.db.internal.domain.type.DbTypeManager;
 import org.mule.module.db.internal.domain.type.ResolvedDbType;
+import org.mule.module.db.internal.domain.type.UnknownDbType;
 import org.mule.module.db.internal.domain.type.UnknownDbTypeException;
 
 import java.sql.ParameterMetaData;
@@ -48,15 +49,24 @@ public class QueryParamTypeResolver implements ParamTypeResolver
             int parameterTypeId = parameterMetaData.getParameterType(queryParam.getIndex());
             String parameterTypeName = parameterMetaData.getParameterTypeName(queryParam.getIndex());
             DbType dbType;
-            try
+            if (parameterTypeName == null)
             {
-                dbType = dbTypeManager.lookup(connection, parameterTypeId, parameterTypeName);
+                // Use unknown data type
+                dbType = UnknownDbType.getInstance();
             }
-            catch (UnknownDbTypeException e)
+            else
             {
-                // Type was not found in the type manager, but the DB knows about it
-                dbType = new ResolvedDbType(parameterTypeId, parameterTypeName);
+                try
+                {
+                    dbType = dbTypeManager.lookup(connection, parameterTypeId, parameterTypeName);
+                }
+                catch (UnknownDbTypeException e)
+                {
+                    // Type was not found in the type manager, but the DB knows about it
+                    dbType = new ResolvedDbType(parameterTypeId, parameterTypeName);
+                }
             }
+
             paramTypes.put(queryParam.getIndex(), dbType);
         }
 
