@@ -1,8 +1,5 @@
 /*
- * $Id: MuleSupportModule.java 22048 2011-05-31 14:39:03Z dfeist $
- * --------------------------------------------------------------------------------------
  * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
- *
  * The software in this package is published under the terms of the CPAL v1.0
  * license, a copy of which has been included with this distribution in the
  * LICENSE.txt file.
@@ -29,8 +26,9 @@ import com.google.inject.spi.TypeEncounter;
 import com.google.inject.spi.TypeListener;
 
 /**
- * TODO
+ * @deprecated Guice module is deprecated and will be removed in Mule 4.
  */
+@Deprecated
 public class MuleSupportModule extends AbstractModule
 {
     protected MuleContext muleContext;
