@@ -23,6 +23,6 @@ public class SimpleCollectionAggregator extends AbstractAggregator
     @Override
     protected EventCorrelatorCallback getCorrelatorCallback(MuleContext muleContext)
     {
-        return new CollectionCorrelatorCallback(muleContext, persistentStores, storePrefix);
+        return new CollectionCorrelatorCallback(muleContext, storePrefix);
     }
 }
